#!/usr/bin/env python3

"""Graphite(Carbon) monitoring relay for rtl_433."""

# Start rtl_433 (rtl_433 -F syslog::1433), then this script

# Option: PEP 3143 - Standard daemon process library
# (use Python 3.x or pip install python-daemon)
# import daemon

from __future__ import print_function
from __future__ import with_statement

import socket
import time
import json

UDP_IP = "127.0.0.1"
UDP_PORT = 1433
GRAPHITE_HOST = "127.0.0.1"
GRAPHITE_PORT = 2003
GRAPHITE_PREFIX = "rtlsdr."


class GraphiteUdpClient(object):
    def __init__(self, host='localhost', port=2003, ipv6=False):
        """Create a new client."""
        fam = socket.AF_INET6 if ipv6 else socket.AF_INET
        family, _, _, _, addr = socket.getaddrinfo(
            host, port, fam, socket.SOCK_DGRAM)[0]
        self._addr = addr
        self._sock = socket.socket(family, socket.SOCK_DGRAM)

    def _send(self, message):
        """Send raw data to graphite."""
        try:
            self._sock.sendto(message, self._addr)
        except (socket.error, RuntimeError):
            pass

    def push(self, path, value, timestamp=None):
        """Send a value to graphite."""
        if not timestamp:
            timestamp = int(time.time())

        message = "{0} {1} {2}".format(path, value, timestamp)
        self._send(message)


sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
sock.bind((UDP_IP, UDP_PORT))


def sanitize(text):
    return text.replace(" ", "_").replace("/", "_").replace(".", "_").replace("&", "")


def parse_syslog(line):
    """Try to extract the payload from a syslog line."""
    line = line.decode("ascii")  # also UTF-8 if BOM
    if line.startswith("<"):
        # fields should be "<PRI>VER", timestamp, hostname, command, pid, mid, sdata, payload
        fields = line.split(None, 7)
        line = fields[-1]
    return line


def rtl_433_probe():
    graphite = GraphiteUdpClient(host=GRAPHITE_HOST,
                                 port=GRAPHITE_PORT)

    while True:
        line, addr = sock.recvfrom(1024)

        try:
            line = parse_syslog(line)
            data = json.loads(line)
            now = int(time.time())

            label = sanitize(data["model"])
            if "channel" in data:
                label += ".CH" + str(data["channel"])
            elif "id" in data:
                label += ".ID" + str(data["id"])
            path = GRAPHITE_PREFIX + label

            if "battery_ok" in data:
                graphite.push(path + '.battery', data["battery_ok"], now)

            if "humidity" in data:
                graphite.push(path + '.humidity', data["humidity"], now)

            graphite.push(path + '.temperature', data["temperature_C"], now)

            # graphite.commit()  # for Pickle protocol only

        except KeyError:
            pass

        except ValueError:
            pass


def run():
    # with daemon.DaemonContext(files_preserve=[sock]):
    #  detach_process=True
    #  uid
    #  gid
    #  working_directory
    rtl_433_probe()


if __name__ == "__main__":
    run()



#!/usr/bin/env python3

"""Statsd monitoring for rtl_433 using pipes."""

# Needs Python statsd Network plugin, s.a. https://github.com/jsocol/pystatsd
#   pip install pystatsd
# -or-
#   curl -o statsd.py https://github.com/jsocol/pystatsd/raw/v3.2/statsd/client.py

import sys
import json
from statsd import StatsClient


def sanitize(text):
    return text.replace(" ", "_")


def rtl_433_probe():
    statsd_host = "127.0.0.1"
    statsd_port = 8125
    statsd_prefix = 'rtlsdr'

    statsd = StatsClient(host=statsd_host,
                         port=statsd_port,
                         prefix=statsd_prefix)

    while True:
        line = sys.stdin.readline()
        if not line:
            break
        try:
            data = json.loads(line)

            label = sanitize(data["model"])
            if "channel" in data:
                label += ".CH" + str(data["channel"])

            if "battery_ok" in data:
                statsd.gauge(label + '.battery', data["battery_ok"])

            if "humidity" in data:
                statsd.gauge(label + '.humidity', data["humidity"])

            statsd.gauge(label + '.temperature', data["temperature_C"])

        except KeyError:
            pass

        except ValueError:
            pass


if __name__ == "__main__":
    rtl_433_probe()



#!/usr/bin/env python3

"""Custom hop controller example for rtl_433's HTTP cmd API."""

# Start rtl_433 (`rtl_433 -F http`), then this script.
# Needs the Requests package to be installed.

import requests
import json
from time import sleep

# You can run rtl_433 and this script on different machines,
# start rtl_433 with `-F http:0.0.0.0`, and change
# to e.g. `HTTP_HOST = "192.168.1.100"` (use your server ip) below.
HTTP_HOST = "127.0.0.1"
HTTP_PORT = 8433


def set_freq(freq):
    return send_cmd({'cmd': 'center_frequency', 'val': freq})


def set_rate(rate):
    return send_cmd({'cmd': 'sample_rate', 'val': rate})


def send_cmd(params):
    url = f'http://{HTTP_HOST}:{HTTP_PORT}/cmd'
    headers = {'Accept': 'application/json'}

    # You will receive JSON events, one per line terminated with CRLF.
    # Use GET
    response = requests.get(url, params=params, headers=headers, timeout=70, stream=True)
    # or POST
    # response = requests.post(url, data=params, headers=headers, timeout=70, stream=True)
    print(f'Sending {params} to {url}')

    # Answer is lines of JSON
    return response.text


def rtl_433_control():
    """Simple timed control of rtl_433 in a loop forever."""

    # Loop forever
    while True:
        try:
            # Set first hop
            sleep(10)
            print(set_freq(433920000))
            print(set_rate(250000))

            # Set second hop
            sleep(10)
            print(set_freq(868000000))
            print(set_rate(1024000))

        except requests.ConnectionError:
            print('Connection failed, retrying in 60s...')
            sleep(60)


if __name__ == "__main__":
    try:
        rtl_433_control()
    except KeyboardInterrupt:
        print('\nExiting.')
        pass



#!/usr/bin/env python3
"""
$ rtl_433 -R 104 -F json | rtl_433_json_to_rtlwmbus.py

A script to convert rtl_433 wmbus json output to rtlwmbus output

Copyright (C) 2019 Benjamin Larsson
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
"""

import sys
import json
import time;

def sanitize(text):
    return text.replace(" ", "_")

def rtl_433_wmbus():
    dup = {}
    seconds = 10
    while True:
        line = sys.stdin.readline()
        ts = int(time.time())
        if not line:
            break

        try:
            event = json.loads(line)

            # Duplicate check + check if dictianary is initialized
            id = int(event['id'])
            if id in dup:
                #print("if %s D:%s T:%s" % (id, dup[id], ts))
                if (dup[id] + seconds) < ts:
                    duplicate = False
                    dup[id] = ts;
                else:
                    duplicate = True
                    #print("Dup! %s" % (id))
            else:
                #print("else %s" % (id))
                dup[id] = ts;
                duplicate = False

            if duplicate != True:
                print("%s1;1;1;%s.000;54;46;%s;0x%s" % (event['mode'], event['time'], event['id'], event['data']) )
                sys.stdout.flush()

        except KeyError:
            pass

        except ValueError:
            pass


if __name__ == "__main__":
    dup_test = {}
    rtl_433_wmbus()



#!/usr/bin/env python3

"""Read events from rtl_433 and gpsd and print out."""

# Needs gpsd (and the Python support from gpsd)
# Start gpsd and rtl_433 (rtl_433 -F syslog::1433), then this script

from __future__ import print_function

import socket
import json
import gps
import threading

# rtl_433 syslog address
UDP_IP = "127.0.0.1"
UDP_PORT = 1433


class GpsPoller(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.gps = gps.gps(mode=gps.WATCH_ENABLE)
        self.running = True

    def run(self):
        while self.running:
            self.gps.next()

    @property
    def utc(self):
        return self.gps.utc

    @property
    def fix(self):
        return self.gps.fix

    @property
    def satellites(self):
        return self.gps.satellites


def parse_syslog(line):
    """Try to extract the payload from a syslog line."""
    line = line.decode("ascii")  # also UTF-8 if BOM
    if line.startswith("<"):
        # fields should be "<PRI>VER", timestamp, hostname, command, pid, mid, sdata, payload
        fields = line.split(None, 7)
        line = fields[-1]
    return line


def prife(label, data, key):
    """Print if exists."""
    if key in data:
        print(label, data[key])



def report_event(data, gpsp):
    """Print out an rtl_433 event with gps data."""

    # don't process if it isn't sensor data
    if "model" not in data:
        return

    # don't process if it isn't TPMS data
    if "type" not in data:
        return
    if data["type"] != "TPMS":
        return

    # now = int(time.time())
    print("----------------------------------------")
    print("Model          ", data["model"])
    prife("ID             ", data, "id")
    prife("Status         ", data, "status")
    prife("State          ", data, "state")
    prife("Flags          ", data, "flags")
    prife("Code           ", data, "code")
    prife("Pressure (kPa) ", data, "pressure_kPa")
    prife("Pressure (PSI) ", data, "pressure_PSI")
    prife("Temperature (C)", data, "temperature_C")
    prife("Temperature (F)", data, "temperature_F")
    print()
    print("latitude       ", gpsp.fix.latitude)
    print("longitude      ", gpsp.fix.longitude)
    print("time utc       ", gpsp.utc, " + ", gpsp.fix.time)
    print("altitude (m)   ", gpsp.fix.altitude)
    print("eps            ", gpsp.fix.eps)
    print("epx            ", gpsp.fix.epx)
    print("epv            ", gpsp.fix.epv)
    print("ept            ", gpsp.fix.ept)
    print("speed (m/s)    ", gpsp.fix.speed)
    print("climb          ", gpsp.fix.climb)
    print("track          ", gpsp.fix.track)
    print("mode           ", gpsp.fix.mode)
    # print("sats           ", gpsp.satellites)


if __name__ == '__main__':
    gpsp = GpsPoller()

    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
    sock.bind((UDP_IP, UDP_PORT))

    try:
        gpsp.start()

        while True:
            line, addr = sock.recvfrom(1024)
            try:
                line = parse_syslog(line)
                data = json.loads(line)
                report_event(data, gpsp)

            except KeyError:
                pass

            except ValueError:
                pass

    except (KeyboardInterrupt, SystemExit): #when you press ctrl+c
        print("\nAborted. Exiting...")
        sock.close()
        gpsp.running = False
        gpsp.join() # wait for the thread to finish

    print("Done.\n")



#!/usr/bin/env python3
""" MQTT test client for receiving rtl_433 JSON data

Example program for receiving and parsing sensor data from rtl_433 sent
as MQTT network messages. Recommended way of sending rtl_433 data on network is:

$ rtl_433 -F json -M utc | mosquitto_pub -t home/rtl_433 -l

An MQTT broker e.g. 'mosquitto' must be running on local computer

Copyright (C) 2017 Tommy Vestermark
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
"""

import datetime
import json
import logging
import multiprocessing as mp
import sys
import time

import paho.mqtt.client as mqtt

MQTT_SERVER = "127.0.0.1"
MQTT_TOPIC_PREFIX = "home/rtl_433"
TIMEOUT_STALE_SENSOR = 600  # Seconds before showing a timeout indicator

# log = logging.getLogger()  # Single process logger
log = mp.log_to_stderr()  # Multiprocessing capable logger
if hasattr(mqtt, 'CallbackAPIVersion'):  # paho >= 2.0.0
    mqtt_client = mqtt.Client(callback_api_version=mqtt.CallbackAPIVersion.VERSION1, client_id="RTL_433_Test")
else:
    mqtt_client = mqtt.Client(client_id="RTL_433_Test")

sensor_state = dict()  # Dictionary containing accumulated sensor state


def print_sensor_state():
    """ Print accumulated sensor state """
    time_now = datetime.datetime.utcnow().replace(microsecond=0)
    print("\nUpdate per {} UTC".format(time_now.isoformat(sep=' ')))
    for model in sensor_state:
        print(model)
        for ID in sensor_state[model]:
            data = sensor_state[model][ID]['data'].copy()
            timestamp = data.pop('time')
            timedelta = (time_now - timestamp).total_seconds()
            indicator = "*" if (timedelta < 2) else "~" if (timedelta > TIMEOUT_STALE_SENSOR) else " "  # Indicator for new and stale data
            print("  ID {:5} {}{} {}".format(ID, timestamp.isoformat(sep=' '), indicator, data))
    sys.stdout.flush()  # Print in real-time


def on_connect(client, userdata, flags, rc):
    """ Callback for when the client receives a CONNACK response from the server. """
    log.info("MQTT Connection: " + mqtt.connack_string(rc))
    if rc != 0:
        log.error("Could not connect. RC: " + str(rc))
        exit()
    # Subscribing in on_connect() means that if we lose the connection and reconnect then subscriptions will be renewed.
    client.subscribe(MQTT_TOPIC_PREFIX)


def on_disconnect(client, userdata, rc):
    if rc != 0:
        log.error("Unexpected disconnection. RC: " + str(rc))


def on_message(client, userdata, msg):
    """ Callback for when a PUBLISH message is received from the server. """
    if msg.topic.startswith(MQTT_TOPIC_PREFIX):
        try:
            # Decode JSON payload
            d = json.loads(msg.payload.decode())
        except json.decoder.JSONDecodeError:
            log.warning("JSON decode error: " + msg.payload.decode())
            return

        # Convert time string to datetime object
        time_str = d.get('time', "0000-00-00 00:00:00")
        time_utc = datetime.datetime.strptime(time_str, "%Y-%m-%d %H:%M:%S")
        d['time'] = time_utc
        # Update sensor_state
        sensor_model = d.pop('model', 'unknown')
        sensor_id = d.pop('id', 0)
        sensor_state.setdefault(sensor_model, {}).setdefault(sensor_id, {})['data'] = d
        print_sensor_state()
    else:
        log.info("Unknown topic: " + msg.topic + "\t" + msg.payload.decode())


# Setup MQTT client
mqtt_client.on_connect = on_connect
mqtt_client.on_disconnect = on_disconnect
mqtt_client.on_message = on_message
mqtt_client.connect(MQTT_SERVER)
mqtt_client.loop_start()


def main():
    """MQTT Test Client"""
    log.setLevel(logging.INFO)
    log.info("MQTT RTL_433 Test Client")

    while True:
        time.sleep(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# coding=utf-8

from __future__ import print_function
from __future__ import with_statement

AP_DESCRIPTION="""
Publish Home Assistant MQTT auto discovery topics for rtl_433 devices.

rtl_433_mqtt_hass.py connects to MQTT and subscribes to the rtl_433
event stream that is published to MQTT by rtl_433. The script publishes
additional MQTT topics that can be used by Home Assistant to automatically
discover and minimally configure new devices.

The configuration topics published by this script tell Home Assistant
what MQTT topics to subscribe to in order to receive the data published
as device topics by MQTT.
"""

AP_EPILOG="""
It is strongly recommended to run rtl_433 with "-C si".
This script requires rtl_433 to publish both event messages and device
messages. If you've changed the device topic in rtl_433, use the same device
topic with the "-T" parameter.

MQTT Username and Password can be set via the cmdline or passed in the
environment: MQTT_USERNAME and MQTT_PASSWORD.

Prerequisites:

1. rtl_433 running separately publishing events and devices messages to MQTT.

2. Python installation
* Python 3.x preferred.
* Needs Paho-MQTT https://pypi.python.org/pypi/paho-mqtt

  Debian/raspbian:  apt install python3-paho-mqtt
  Or
  pip install paho-mqtt
* Optional for running as a daemon see PEP 3143 - Standard daemon process library
  (use Python 3.x or pip install python-daemon)


Running:

This script can run continually as a daemon, where it will publish
a configuration topic for the device events sent to MQTT by rtl_433
every 10 minutes.

Alternatively if the rtl_433 devices in your environment change infrequently
this script can use the MQTT retain flag to make the configuration topics
persistent. The script will only need to be run when things change or if
the MQTT server loses its retained messages.

Getting rtl_433 devices back after Home Assistant restarts will happen
more quickly if MQTT retain is enabled. Note however that definitions
for any transitient devices/false positives will retained indefinitely.

If your sensor values change infrequently and you prefer to write the most
recent value even if not changed set -f to append "force_update = true" to
all configs. This is useful if you're graphing the sensor data or want to
alert on missing data.

If you have changed the topic structure from the default topics in the rtl433
configuration use the -T parameter to set the same topic structure here.

Suggestions:

Running this script will cause a number of Home Assistant entities (sensors
and binary sensors) to be created. These entities can linger for a while unless
the topic is republished with an empty config string.  To avoid having to
do a lot of clean up When running this initially or debugging, set this
script to publish to a topic other than the one Home Assistant users (homeassistant).

MQTT Explorer (http://mqtt-explorer.com/) is a very nice GUI for
working with MQTT. It is free, cross platform, and OSS. The structured
hierarchical view makes it easier to understand what rtl_433 is publishing
and how this script works with Home Assistant.

MQTT Explorer also makes it easy to publish an empty config topic to delete an
entity from Home Assistant.


As of 2020-10, Home Assistant MQTT auto discovery doesn't currently support
supplying "friendly name", and "area" key, so some configuration must be
done in Home Assistant.

There is a single global set of field mappings to Home Assistant meta data.

"""



# import daemon


import os
import argparse
import logging
import time
import json
import paho.mqtt.client as mqtt
import re


discovery_timeouts = {}

# Fields that get ignored when publishing to Home Assistant
# (reduces noise to help spot missing field mappings)
SKIP_KEYS = [ "type", "model", "subtype", "channel", "id", "mic", "mod",
                "freq", "sequence_num", "message_type", "exception", "raw_msg" ]


# Global mapping of rtl_433 field names to Home Assistant metadata.
# @todo - should probably externalize to a config file
# @todo - Model specific definitions might be needed

mappings = {
    "temperature_C": {
        "device_type": "sensor",
        "object_suffix": "T",
        "config": {
            "device_class": "temperature",
            "name": "Temperature",
            "unit_of_measurement": "°C",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },
    "temperature_1_C": {
        "device_type": "sensor",
        "object_suffix": "T1",
        "config": {
            "device_class": "temperature",
            "name": "Temperature 1",
            "unit_of_measurement": "°C",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },
    "temperature_2_C": {
        "device_type": "sensor",
        "object_suffix": "T2",
        "config": {
            "device_class": "temperature",
            "name": "Temperature 2",
            "unit_of_measurement": "°C",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },
    "temperature_3_C": {
        "device_type": "sensor",
        "object_suffix": "T3",
        "config": {
            "device_class": "temperature",
            "name": "Temperature 3",
            "unit_of_measurement": "°C",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },
    "temperature_4_C": {
        "device_type": "sensor",
        "object_suffix": "T4",
        "config": {
            "device_class": "temperature",
            "name": "Temperature 4",
            "unit_of_measurement": "°C",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },
    "temperature_F": {
        "device_type": "sensor",
        "object_suffix": "F",
        "config": {
            "device_class": "temperature",
            "name": "Temperature",
            "unit_of_measurement": "°F",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },

    # This diagnostic sensor is useful to see when a device last sent a value,
    # even if the value didn't change.
    # https://community.home-assistant.io/t/send-metrics-to-influxdb-at-regular-intervals/9096
    # https://github.com/home-assistant/frontend/discussions/13687
    "time": {
        "device_type": "sensor",
        "object_suffix": "UTC",
        "config": {
            "device_class": "timestamp",
            "name": "Timestamp",
            "entity_category": "diagnostic",
            "enabled_by_default": False,
            "icon": "mdi:clock-in"
        }
    },

    "battery_ok": {
        "device_type": "sensor",
        "object_suffix": "B",
        "config": {
            "device_class": "battery",
            "name": "Battery",
            "unit_of_measurement": "%",
            "value_template": "{{ ((float(value) * 99)|round(0)) + 1 }}",
            "state_class": "measurement",
            "entity_category": "diagnostic"
        }
    },

    "battery_mV": {
        "device_type": "sensor",
        "object_suffix": "mV",
        "config": {
            "device_class": "voltage",
            "name": "Battery mV",
            "unit_of_measurement": "mV",
            "value_template": "{{ float(value) }}",
            "state_class": "measurement",
            "entity_category": "diagnostic"
        }
    },

    "supercap_V": {
        "device_type": "sensor",
        "object_suffix": "V",
        "config": {
            "device_class": "voltage",
            "name": "Supercap V",
            "unit_of_measurement": "V",
            "value_template": "{{ float(value) }}",
            "state_class": "measurement",
            "entity_category": "diagnostic"
        }
    },

    "humidity": {
        "device_type": "sensor",
        "object_suffix": "H",
        "config": {
            "device_class": "humidity",
            "name": "Humidity",
            "unit_of_measurement": "%",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },
    "humidity_1": {
        "device_type": "sensor",
        "object_suffix": "H1",
        "config": {
            "device_class": "humidity",
            "name": "Humidity 1",
            "unit_of_measurement": "%",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },
    "humidity_2": {
        "device_type": "sensor",
        "object_suffix": "H2",
        "config": {
            "device_class": "humidity",
            "name": "Humidity 2",
            "unit_of_measurement": "%",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "moisture": {
        "device_type": "sensor",
        "object_suffix": "M",
        "config": {
            "device_class": "moisture",
            "name": "Moisture",
            "unit_of_measurement": "%",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "detect_wet": {
        "device_type": "binary_sensor",
        "object_suffix": "moisture",
        "config": {
            "name": "Water Sensor",
            "device_class": "moisture",
            "force_update": "true",
            "payload_on": "1",
            "payload_off": "0"
        }
    },

    "pressure_hPa": {
        "device_type": "sensor",
        "object_suffix": "P",
        "config": {
            "device_class": "pressure",
            "name": "Pressure",
            "unit_of_measurement": "hPa",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "pressure_kPa": {
        "device_type": "sensor",
        "object_suffix": "P",
        "config": {
            "device_class": "pressure",
            "name": "Pressure",
            "unit_of_measurement": "kPa",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "wind_speed_km_h": {
        "device_type": "sensor",
        "object_suffix": "WS",
        "config": {
            "device_class": "wind_speed",
            "name": "Wind Speed",
            "unit_of_measurement": "km/h",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "wind_avg_km_h": {
        "device_type": "sensor",
        "object_suffix": "WS",
        "config": {
            "device_class": "wind_speed",
            "name": "Wind Speed",
            "unit_of_measurement": "km/h",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "wind_avg_mi_h": {
        "device_type": "sensor",
        "object_suffix": "WS",
        "config": {
            "device_class": "wind_speed",
            "name": "Wind Speed",
            "unit_of_measurement": "mph",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "wind_avg_m_s": {
        "device_type": "sensor",
        "object_suffix": "WS",
        "config": {
            "device_class": "wind_speed",
            "name": "Wind Average",
            "unit_of_measurement": "km/h",
            "value_template": "{{ (float(value|float) * 3.6) | round(2) }}",
            "state_class": "measurement"
        }
    },

    "wind_speed_m_s": {
        "device_type": "sensor",
        "object_suffix": "WS",
        "config": {
            "device_class": "wind_speed",
            "name": "Wind Speed",
            "unit_of_measurement": "km/h",
            "value_template": "{{ float(value|float) * 3.6 }}",
            "state_class": "measurement"
        }
    },

    "gust_speed_km_h": {
        "device_type": "sensor",
        "object_suffix": "GS",
        "config": {
            "device_class": "wind_speed",
            "name": "Gust Speed",
            "unit_of_measurement": "km/h",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "wind_max_km_h": {
        "device_type": "sensor",
        "object_suffix": "GS",
        "config": {
            "device_class": "wind_speed",
            "name": "Wind max speed",
            "unit_of_measurement": "km/h",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "wind_max_m_s": {
        "device_type": "sensor",
        "object_suffix": "GS",
        "config": {
            "device_class": "wind_speed",
            "name": "Wind max",
            "unit_of_measurement": "km/h",
            "value_template": "{{ (float(value|float) * 3.6) | round(2) }}",
            "state_class": "measurement"
        }
    },

    "gust_speed_m_s": {
        "device_type": "sensor",
        "object_suffix": "GS",
        "config": {
            "device_class": "wind_speed",
            "name": "Gust Speed",
            "unit_of_measurement": "km/h",
            "value_template": "{{ float(value|float) * 3.6 }}",
            "state_class": "measurement"
        }
    },

    "wind_dir_deg": {
        "device_type": "sensor",
        "object_suffix": "WD",
        "config": {
            "name": "Wind Direction",
            "unit_of_measurement": "°",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "rain_mm": {
        "device_type": "sensor",
        "object_suffix": "RT",
        "config": {
            "device_class": "precipitation",
            "name": "Rain Total",
            "unit_of_measurement": "mm",
            "value_template": "{{ value|float|round(2) }}",
            "state_class": "total_increasing"
        }
    },

    "rain_rate_mm_h": {
        "device_type": "sensor",
        "object_suffix": "RR",
        "config": {
            "device_class": "precipitation_intensity",
            "name": "Rain Rate",
            "unit_of_measurement": "mm/h",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "rain_in": {
        "device_type": "sensor",
        "object_suffix": "RT",
        "config": {
            "device_class": "precipitation",
            "name": "Rain Total",
            "unit_of_measurement": "in",
            "value_template": "{{ value|float|round(2) }}",
            "state_class": "total_increasing"
        }
    },

    "rain_rate_in_h": {
        "device_type": "sensor",
        "object_suffix": "RR",
        "config": {
            "device_class": "precipitation_intensity",
            "name": "Rain Rate",
            "unit_of_measurement": "in/h",
            "value_template": "{{ value|float|round(2) }}",
            "state_class": "measurement"
        }
    },

    "reed_open": {
        "device_type": "binary_sensor",
        "object_suffix": "reed_open",
        "config": {
            "device_class": "safety",
            "force_update": "true",
            "payload_on": "1",
            "payload_off": "0",
            "entity_category": "diagnostic"
        }
    },

    "contact_open": {
        "device_type": "binary_sensor",
        "object_suffix": "contact_open",
        "config": {
            "device_class": "safety",
            "force_update": "true",
            "payload_on": "1",
            "payload_off": "0",
            "entity_category": "diagnostic"
        }
    },

    "tamper": {
        "device_type": "binary_sensor",
        "object_suffix": "tamper",
        "config": {
            "device_class": "safety",
            "force_update": "true",
            "payload_on": "1",
            "payload_off": "0",
            "entity_category": "diagnostic"
        }
    },

    "alarm": {
        "device_type": "binary_sensor",
        "object_suffix": "alarm",
        "config": {
            "device_class": "safety",
            "force_update": "true",
            "payload_on": "1",
            "payload_off": "0",
            "entity_category": "diagnostic"
        }
    },

    "rssi": {
        "device_type": "sensor",
        "object_suffix": "rssi",
        "config": {
            "device_class": "signal_strength",
            "unit_of_measurement": "dB",
            "value_template": "{{ value|float|round(2) }}",
            "state_class": "measurement",
            "entity_category": "diagnostic"
        }
    },

    "snr": {
        "device_type": "sensor",
        "object_suffix": "snr",
        "config": {
            "device_class": "signal_strength",
            "unit_of_measurement": "dB",
            "value_template": "{{ value|float|round(2) }}",
            "state_class": "measurement",
            "entity_category": "diagnostic"
        }
    },

    "noise": {
        "device_type": "sensor",
        "object_suffix": "noise",
        "config": {
            "device_class": "signal_strength",
            "unit_of_measurement": "dB",
            "value_template": "{{ value|float|round(2) }}",
            "state_class": "measurement",
            "entity_category": "diagnostic"
        }
    },

    "depth_cm": {
        "device_type": "sensor",
        "object_suffix": "D",
        "config": {
            "name": "Depth",
            "unit_of_measurement": "cm",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "power_W": {
        "device_type": "sensor",
        "object_suffix": "watts",
        "config": {
            "device_class": "power",
            "name": "Power",
            "unit_of_measurement": "W",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "energy_kWh": {
        "device_type": "sensor",
        "object_suffix": "kwh",
        "config": {
            "device_class": "energy",
            "name": "Energy",
            "unit_of_measurement": "kWh",
            "value_template": "{{ value|float }}",
            "state_class": "total_increasing"
        }
    },

    "current_A": {
        "device_type": "sensor",
        "object_suffix": "A",
        "config": {
            "device_class": "current",
            "name": "Current",
            "unit_of_measurement": "A",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "voltage_V": {
        "device_type": "sensor",
        "object_suffix": "V",
        "config": {
            "device_class": "voltage",
            "name": "Voltage",
            "unit_of_measurement": "V",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    "light_lux": {
        "device_type": "sensor",
        "object_suffix": "lux",
        "config": {
            "device_class": "illuminance",
            "name": "Outside Luminance",
            "unit_of_measurement": "lx",
            "value_template": "{{ value|int }}",
            "state_class": "measurement"
        }
    },
    "lux": {
        "device_type": "sensor",
        "object_suffix": "lux",
        "config": {
            "device_class": "illuminance",
            "name": "Outside Luminance",
            "unit_of_measurement": "lx",
            "value_template": "{{ value|int }}",
            "state_class": "measurement"
        }
    },

    "uv": {
        "device_type": "sensor",
        "object_suffix": "uv",
        "config": {
            "name": "UV Value",
            "unit_of_measurement": "UV",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },
    "uvi": {
        "device_type": "sensor",
        "object_suffix": "uvi",
        "config": {
            "name": "UV Index",
            "unit_of_measurement": "UV Index",
            "value_template": "{{ value|float|round(1) }}",
            "state_class": "measurement"
        }
    },

    "storm_dist_km": {
        "device_type": "sensor",
        "object_suffix": "stdist",
        "config": {
            "name": "Lightning Distance",
            "unit_of_measurement": "km",
            "value_template": "{{ value|int }}",
            "state_class": "measurement"
        }
    },

    "storm_dist": {
        "device_type": "sensor",
        "object_suffix": "stdist",
        "config": {
            "name": "Lightning Distance",
            "unit_of_measurement": "mi",
            "value_template": "{{ value|int }}",
            "state_class": "measurement"
        }
    },

    "strike_distance": {
        "device_type": "sensor",
        "object_suffix": "stdist",
        "config": {
            "name": "Lightning Distance",
            "unit_of_measurement": "mi",
            "value_template": "{{ value|int }}",
            "state_class": "measurement"
        }
    },

    "strike_count": {
        "device_type": "sensor",
        "object_suffix": "strcnt",
        "config": {
            "name": "Lightning Strike Count",
            "value_template": "{{ value|int }}",
            "state_class": "total_increasing"
        }
    },

    "consumption_data": {
        "device_type": "sensor",
        "object_suffix": "consumption",
        "config": {
            "name": "SCM Consumption Value",
            "value_template": "{{ value|int }}",
            "state_class": "total_increasing",
        }
    },

    "consumption": {
        "device_type": "sensor",
        "object_suffix": "consumption",
        "config": {
            "name": "SCMplus Consumption Value",
            "value_template": "{{ value|int }}",
            "state_class": "total_increasing",
        }
    },

    "channel": {
        "device_type": "device_automation",
        "object_suffix": "CH",
        "config": {
           "automation_type": "trigger",
           "type": "button_short_release",
           "subtype": "button_1",
        }
    },

    "button": {
        "device_type": "device_automation",
        "object_suffix": "BTN",
        "config": {
           "automation_type": "trigger",
           "type": "button_short_release",
           "subtype": "button_2",
        }
    },

    # WH45, WH290
    "pm2_5_ug_m3": {
        "device_type": "sensor",
        "object_suffix": "PM25",
        "config": {
            "device_class": "pm25",
            "name": "PM 2.5 Concentration",
            "unit_of_measurement": "µg/m³",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    # WH45
    "pm10_ug_m3": {
        "device_type": "sensor",
        "object_suffix": "PM10",
        "config": {
            "device_class": "pm10",
            "name": "PM 10 Concentration",
            "unit_of_measurement": "µg/m³",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    # WH290
    "estimated_pm10_0_ug_m3": {
        "device_type": "sensor",
        "object_suffix": "PM10",
        "config": {
            "device_class": "pm10",
            "name": "Estimated PM 10 Concentration",
            "unit_of_measurement": "µg/m³",
            "value_template": "{{ value|float }}",
            "state_class": "measurement"
        }
    },

    # WH45
    "co2_ppm": {
        "device_type": "sensor",
        "object_suffix": "CO2",
        "config": {
            "device_class": "carbon_dioxide",
            "name": "CO2 Concentration",
            "unit_of_measurement": "ppm",
            "value_template": "{{ value|int }}",
            "state_class": "measurement"
        }
    },

    "ext_power": {
        "device_type": "binary_sensor",
        "object_suffix": "extpwr",
        "config": {
            "device_class": "power",
            "name": "External Power",
            "payload_on": "1",
            "payload_off": "0",
            "entity_category": "diagnostic"
        }
    },

}

# Use secret_knock to trigger device automations for Honeywell ActivLink
# doorbells. We have this outside of mappings as we need to configure two
# different configuration topics.
secret_knock_mappings = [

    {
        "device_type": "device_automation",
        "object_suffix": "Knock",
        "config": {
            "automation_type": "trigger",
            "type": "button_short_release",
            "subtype": "button_1",
            "payload": 0,
        }
    },

    {
        "device_type": "device_automation",
        "object_suffix": "Secret-Knock",
        "config": {
            "automation_type": "trigger",
            "type": "button_triple_press",
            "subtype": "button_1",
            "payload": 1,
        }
    },

]

TOPIC_PARSE_RE = re.compile(r'\[(?P<slash>/?)(?P<token>[^\]:]+):?(?P<default>[^\]:]*)\]')

def mqtt_connect(client, userdata, flags, rc):
    """Callback for MQTT connects."""

    logging.info("MQTT connected: " + mqtt.connack_string(rc))
    if rc != 0:
        logging.error("Could not connect. Error: " + str(rc))
    else:
        logging.info("Subscribing to: " + args.rtl_topic)
        client.subscribe(args.rtl_topic)


def mqtt_disconnect(client, userdata, rc):
    """Callback for MQTT disconnects."""
    logging.info("MQTT disconnected: " + mqtt.connack_string(rc))


def mqtt_message(client, userdata, msg):
    """Callback for MQTT message PUBLISH."""
    logging.debug("MQTT message: " + json.dumps(msg.payload.decode()))

    try:
        # Decode JSON payload
        data = json.loads(msg.payload.decode())

    except json.decoder.JSONDecodeError:
        logging.error("JSON decode error: " + msg.payload.decode())
        return

    topicprefix = "/".join(msg.topic.split("/", 2)[:2])
    bridge_event_to_hass(client, topicprefix, data)


def sanitize(text):
    """Sanitize a name for Graphite/MQTT use."""
    return (text
            .replace(" ", "_")
            .replace("/", "_")
            .replace(".", "_")
            .replace("&", ""))

def rtl_433_device_info(data, topic_prefix):
    """Return rtl_433 device topic to subscribe to for a data element, based on the
    rtl_433 device topic argument, as well as the device identifier"""

    path_elements = []
    id_elements = []
    last_match_end = 0
    # The default for args.device_topic_suffix is the same topic structure
    # as set by default in rtl433 config
    for match in re.finditer(TOPIC_PARSE_RE, args.device_topic_suffix):
        path_elements.append(args.device_topic_suffix[last_match_end:match.start()])
        key = match.group(2)
        if key in data:
            # If we have this key, prepend a slash if needed
            if match.group(1):
                path_elements.append('/')
            element = sanitize(str(data[key]))
            path_elements.append(element)
            id_elements.append(element)
        elif match.group(3):
            path_elements.append(match.group(3))
        last_match_end = match.end()

    path = ''.join(list(filter(lambda item: item, path_elements)))
    id = '-'.join(id_elements)
    return (f"{topic_prefix}/{path}", id)


def publish_config(mqttc, topic, model, object_id, mapping, key=None):
    """Publish Home Assistant auto discovery data."""
    global discovery_timeouts

    device_type = mapping["device_type"]
    object_suffix = mapping["object_suffix"]
    object_name = "-".join([object_id, object_suffix])

    path = "/".join([args.discovery_prefix, device_type, object_id, object_name, "config"])

    # check timeout
    now = time.time()
    if path in discovery_timeouts:
        if discovery_timeouts[path] > now:
            logging.debug("Discovery timeout in the future for: " + path)
            return False

    discovery_timeouts[path] = now + args.discovery_interval

    config = mapping["config"].copy()

    # Device Automation configuration is in a different structure compared to
    # all other mqtt discovery types.
    # https://www.home-assistant.io/integrations/device_trigger.mqtt/
    if device_type == 'device_automation':
        config["topic"] = topic
        config["platform"] = 'mqtt'
    else:
        readable_name = mapping["config"]["name"] if "name" in mapping["config"] else key
        config["state_topic"] = topic
        config["unique_id"] = object_name
        config["name"] = readable_name
    config["device"] = { "identifiers": [object_id], "name": object_id, "model": model, "manufacturer": "rtl_433" }

    if args.force_update:
        config["force_update"] = "true"

    if args.expire_after:
        config["expire_after"] = args.expire_after

    logging.debug(path + ":" + json.dumps(config))

    mqttc.publish(path, json.dumps(config), retain=args.retain)

    return True

def bridge_event_to_hass(mqttc, topic_prefix, data):
    """Translate some rtl_433 sensor data to Home Assistant auto discovery."""

    if "model" not in data:
        # not a device event
        logging.debug("Model is not defined. Not sending event to Home Assistant.")
        return

    model = sanitize(data["model"])

    skipped_keys = []
    published_keys = []

    base_topic, device_id = rtl_433_device_info(data, topic_prefix)
    if not device_id:
        # no unique device identifier
        logging.warning("No suitable identifier found for model: %s", model)
        return

    if args.ids and "id" in data and data.get("id") not in args.ids:
        # not in the safe list
        logging.debug("Device (%s) is not in the desired list of device ids: [%s]" % (data["id"], ids))
        return

    # detect known attributes
    for key in data.keys():
        if key in mappings:
            # topic = "/".join([topicprefix,"devices",model,instance,key])
            topic = "/".join([base_topic, key])
            if publish_config(mqttc, topic, model, device_id, mappings[key], key):
                published_keys.append(key)
        else:
            if key not in SKIP_KEYS:
                skipped_keys.append(key)

    if "secret_knock" in data.keys():
        for m in secret_knock_mappings:
            topic = "/".join([base_topic, "secret_knock"])
            if publish_config(mqttc, topic, model, device_id, m, "secret_knock"):
                published_keys.append("secret_knock")

    if published_keys:
        logging.info("Published %s: %s" % (device_id, ", ".join(published_keys)))

        if skipped_keys:
            logging.info("Skipped %s: %s" % (device_id, ", ".join(skipped_keys)))


def rtl_433_bridge():
    """Run a MQTT Home Assistant auto discovery bridge for rtl_433."""

    if hasattr(mqtt, 'CallbackAPIVersion'):  # paho >= 2.0.0
        mqttc = mqtt.Client(callback_api_version=mqtt.CallbackAPIVersion.VERSION1)
    else:
        mqttc = mqtt.Client()

    if args.debug:
        mqttc.enable_logger()

    if args.user is not None:
        mqttc.username_pw_set(args.user, args.password)

    if args.ca_cert is not None:
        mqttc.tls_set(certfile=args.cert, keyfile=args.key, ca_certs=args.ca_cert)

    mqttc.on_connect = mqtt_connect
    mqttc.on_disconnect = mqtt_disconnect
    mqttc.on_message = mqtt_message
    mqttc.connect_async(args.host, args.port, 60)
    logging.debug("MQTT Client: Starting Loop")
    mqttc.loop_start()

    while True:
        time.sleep(1)


def run():
    """Run main or daemon."""
    # with daemon.DaemonContext(files_preserve=[sock]):
    #  detach_process=True
    #  uid
    #  gid
    #  working_directory
    rtl_433_bridge()


if __name__ == "__main__":
    logging.basicConfig(format='[%(asctime)s] %(levelname)s:%(name)s:%(message)s',datefmt='%Y-%m-%dT%H:%M:%S%z')
    logging.getLogger().setLevel(logging.INFO)

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=AP_DESCRIPTION,
                                     epilog=AP_EPILOG)

    parser.add_argument("-d", "--debug", action="store_true")
    parser.add_argument("-q", "--quiet", action="store_true")
    parser.add_argument("-u", "--user", type=str, help="MQTT username")
    parser.add_argument("-P", "--password", type=str, help="MQTT password")
    parser.add_argument("-H", "--host", type=str, default="127.0.0.1",
                        help="MQTT hostname to connect to (default: %(default)s)")
    parser.add_argument("-p", "--port", type=int, default=1883,
                        help="MQTT port (default: %(default)s)")
    parser.add_argument("-c", "--ca_cert", type=str, help="MQTT TLS CA certificate path")
    parser.add_argument("--cert", type=str, help="MQTT TLS certificate path")
    parser.add_argument("--key", type=str, help="MQTT TLS certificate key path")
    parser.add_argument("-r", "--retain", action="store_true")
    parser.add_argument("-f", "--force_update", action="store_true",
                        help="Append 'force_update = true' to all configs.")
    parser.add_argument("-R", "--rtl-topic", type=str,
                        default="rtl_433/+/events",
                        dest="rtl_topic",
                        help="rtl_433 MQTT event topic to subscribe to (default: %(default)s)")
    parser.add_argument("-D", "--discovery-prefix", type=str,
                        dest="discovery_prefix",
                        default="homeassistant",
                        help="Home Assistant MQTT topic prefix (default: %(default)s)")
    # This defaults to the rtl433 config default, so we assemble the same topic structure
    parser.add_argument("-T", "--device-topic_suffix", type=str,
                        dest="device_topic_suffix",
                        default="devices[/type][/model][/subtype][/channel][/id]",
                        help="rtl_433 device topic suffix (default: %(default)s)")
    parser.add_argument("-i", "--interval", type=int,
                        dest="discovery_interval",
                        default=600,
                        help="Interval to republish config topics in seconds (default: %(default)d)")
    parser.add_argument("-x", "--expire-after", type=int,
                        dest="expire_after",
                        help="Number of seconds with no updates after which the sensor becomes unavailable")
    parser.add_argument("-I", "--ids", type=int, nargs="+",
                        help="ID's of devices that will be discovered (omit for all)")
    args = parser.parse_args()

    if args.debug and args.quiet:
        logging.critical("Debug and quiet can not be specified at the same time")
        exit(1)

    if args.debug:
        logging.info("Enabling debug logging")
        logging.getLogger().setLevel(logging.DEBUG)
    if args.quiet:
        logging.getLogger().setLevel(logging.ERROR)

    # allow setting MQTT username and password via environment variables
    if not args.user and 'MQTT_USERNAME' in os.environ:
        args.user = os.environ['MQTT_USERNAME']

    if not args.password and 'MQTT_PASSWORD' in os.environ:
        args.password = os.environ['MQTT_PASSWORD']

    if not args.user or not args.password:
        logging.warning("User or password is not set. Check credentials if subscriptions do not return messages.")

    if args.ids:
        ids = ', '.join(str(id) for id in args.ids)
        logging.info("Only discovering devices with ids: [%s]" % ids)
    else:
        logging.info("Discovering all devices")

    run()



#!/usr/bin/env python3

"""Helper command for rtl_433 to visualize a sample file in a web browser."""

from http.server import BaseHTTPRequestHandler, HTTPServer
import sys
import subprocess
import webbrowser

hostName = "localhost"
serverPort = 8080


def parseToPulseData(filename):
    ret = subprocess.run(["rtl_433", "-F", "null", "-w", "OOK:-", filename], capture_output=True)
    return ret.stdout


class PulseServer(BaseHTTPRequestHandler):
    def do_GET(self):
        global pulsedata

        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()

        self.wfile.write(bytes("""
<!doctype html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width,initial-scale=1">
    <meta name="mobile-web-app-capable" content="yes">
    <meta name="apple-mobile-web-app-capable" content="yes">
    <link rel="apple-touch-icon" href="https://triq.org/pdv/icon.png" />
    <link rel="apple-touch-icon" sizes="76x76" href="https://triq.org/pdv/icon.76.png" />
    <link rel="apple-touch-icon" sizes="120x120" href="https://triq.org/pdv/icon.120.png" />
    <link rel="apple-touch-icon" sizes="152x152" href="https://triq.org/pdv/icon.152.png" />
    <link rel="apple-touch-icon" sizes="180x180" href="https://triq.org/pdv/icon.180.png" />
    <link rel="icon" sizes="192x192" href="https://triq.org/pdv/icon.192.png">
    <link rel="icon" sizes="128x128" href="https://triq.org/pdv/icon.128.png">
    <link rel="icon" href="https://triq.org/pdv/favicon.ico">
    <meta name="description" content="I/Q Spectrogram and Pulsedata.">
    <title>I/Q Spectrogram &amp; Pulsedata</title>
    <link href="https://triq.org/pdv/css/app.css" rel="stylesheet">
</head>
<body>
    <noscript><strong>We're sorry but I/Q Spectrogram &amp; Pulsedata doesn't work properly without JavaScript
      enabled. Please enable it to continue.</strong></noscript>
    <div id="app"></div>
<script>
window.pulseData=`
""", "utf-8"))
        self.wfile.write(pulsedata)
        self.wfile.write(bytes("""
`
</script>
<script type="module" src="https://triq.org/pdv/js/app.js"></script>
</body>
</html>
""", "utf-8"))


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage:\n%s FILENAME.cu8" % (sys.argv[0]))
        exit(1)

    filename = sys.argv[1]
    pulsedata = parseToPulseData(filename)

    while serverPort < 65536:
        try:
            webServer = HTTPServer((hostName, serverPort), PulseServer)
            break;
        except OSError as e:
            if e.errno != 48 or serverPort >= 65535:
                raise
            serverPort += 1  # Address already in use

    print("If the browser doesn't open go to http://%s:%s" % (hostName, serverPort))

    try:
        webbrowser.open("http://%s:%s/" % (hostName, serverPort))
        webServer.handle_request()  # once
    except KeyboardInterrupt:
        pass

    webServer.server_close()
    print("done.")



#!/usr/bin/env python3

"""Custom data handling example for rtl_433."""

# Start rtl_433 (rtl_433 -F syslog::1433), then this script

from __future__ import print_function

import socket
import json

# You can run rtl_433 and this script on different machines,
# start rtl_433 with `-F syslog:YOURTARGETIP:1433`, and change
# to `UDP_IP = "0.0.0.0"` (listen to the whole network) below.
UDP_IP = "127.0.0.1"
UDP_PORT = 1433


def parse_syslog(line):
    """Try to extract the payload from a syslog line."""
    line = line.decode("ascii")  # also UTF-8 if BOM
    if line.startswith("<"):
        # fields should be "<PRI>VER", timestamp, hostname, command, pid, mid, sdata, payload
        fields = line.split(None, 7)
        line = fields[-1]
    return line


def rtl_433_listen():
    """Listen to all messages in a loop forever."""
    # Open a UDP socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    # Bind the UDP socket to a listening address
    sock.bind((UDP_IP, UDP_PORT))

    # Loop forever
    while True:
        # Receive a message
        line, addr = sock.recvfrom(1024)

        try:
            # Parse the message format
            line = parse_syslog(line)
            # Decode the message as JSON
            data = json.loads(line)

            #
            # Change for your custom handling below, this is a simple example
            #
            label = data["model"]
            if "channel" in data:
                label += ".CH" + str(data["channel"])
            elif "id" in data:
                label += ".ID" + str(data["id"])

            # E.g. match `model` and `id` to a descriptive name.
            if data["model"] == "LaCrosse-TX" and data["id"] == 123:
                label = "Living Room"

            if "battery_ok" in data:
                if data["battery_ok"] == 0:
                    print(label + ' Battery empty!')

            if "temperature_C" in data:
                print(label + ' Temperature ', data["temperature_C"])

            if "humidity" in data:
                print(label + ' Humidity ', data["humidity"])

            # Ignore unknown message data and continue
        except KeyError:
            pass

        except ValueError:
            pass


if __name__ == "__main__":
    rtl_433_listen()



#!/usr/bin/env python3

"""Custom data handling example for rtl_433's HTTP (line) streaming API of JSON events."""

# Start rtl_433 (`rtl_433 -F http`), then this script.
# Needs the Requests package to be installed.

import requests
import json
from time import sleep

# You can run rtl_433 and this script on different machines,
# start rtl_433 with `-F http:0.0.0.0`, and change
# to e.g. `HTTP_HOST = "192.168.1.100"` (use your server ip) below.
HTTP_HOST = "127.0.0.1"
HTTP_PORT = 8433


def stream_lines():
    url = f'http://{HTTP_HOST}:{HTTP_PORT}/stream'
    headers = {'Accept': 'application/json'}

    # You will receive JSON events, one per line terminated with CRLF.
    # On Events and Stream endpoints a keep-alive of CRLF will be send every 60 seconds.
    response = requests.get(url, headers=headers, timeout=70, stream=True)
    print(f'Connected to {url}')

    for chunk in response.iter_lines():
        yield chunk


def handle_event(line):
    try:
        # Decode the message as JSON
        data = json.loads(line)

        #
        # Change for your custom handling below, this is a simple example
        #
        label = data["model"]
        if "channel" in data:
            label += ".CH" + str(data["channel"])
        elif "id" in data:
            label += ".ID" + str(data["id"])

        # E.g. match `model` and `id` to a descriptive name.
        if data["model"] == "LaCrosse-TX" and data["id"] == 123:
            label = "Living Room"

        if "battery_ok" in data:
            if data["battery_ok"] == 0:
                print(label + ' Battery empty!')

        if "temperature_C" in data:
            print(label + ' Temperature ', data["temperature_C"])

        if "humidity" in data:
            print(label + ' Humidity ', data["humidity"])

    except KeyError:
        # Ignore unknown message data and continue
        pass

    except ValueError as e:
        # Warn on decoding errors
        print(f'Event format not recognized: {e}')


def rtl_433_listen():
    """Listen to all messages in a loop forever."""

    # Loop forever
    while True:
        try:
            # Open the HTTP (line) streaming API of JSON events
            for chunk in stream_lines():
                # print(chunk)
                chunk = chunk.rstrip()
                if not chunk:
                    # filter out keep-alive empty lines
                    continue
                # Decode the JSON message
                handle_event(chunk)

        except requests.ConnectionError:
            print('Connection failed, retrying...')
            sleep(5)


if __name__ == "__main__":
    try:
        rtl_433_listen()
    except KeyboardInterrupt:
        print('\nExiting.')
        pass



#!/usr/bin/env python3

"""Statsd monitoring relay for rtl_433."""

# Needs Python statsd Network plugin, s.a. https://github.com/jsocol/pystatsd
#   pip install pystatsd
# -or-
#   curl -o statsd.py https://github.com/jsocol/pystatsd/raw/v3.2/statsd/client.py
# Start rtl_433 (rtl_433 -F syslog::1433), then this script

from __future__ import print_function

import socket
import json
from statsd import StatsClient

UDP_IP = "127.0.0.1"
UDP_PORT = 1433
STATSD_HOST = "127.0.0.1"
STATSD_PORT = 8125
STATSD_PREFIX = "rtlsdr"

sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
sock.bind((UDP_IP, UDP_PORT))


def sanitize(text):
    return text.replace(" ", "_")


def parse_syslog(line):
    """Try to extract the payload from a syslog line."""
    line = line.decode("ascii")  # also UTF-8 if BOM
    if line.startswith("<"):
        # fields should be "<PRI>VER", timestamp, hostname, command, pid, mid, sdata, payload
        fields = line.split(None, 7)
        line = fields[-1]
    return line


def rtl_433_probe():
    statsd = StatsClient(host=STATSD_HOST,
                         port=STATSD_PORT,
                         prefix=STATSD_PREFIX)

    while True:
        line, addr = sock.recvfrom(1024)

        try:
            line = parse_syslog(line)
            data = json.loads(line)

            label = sanitize(data["model"])
            if "channel" in data:
                label += ".CH" + str(data["channel"])

            if "battery_ok" in data:
                statsd.gauge(label + '.battery', data["battery_ok"])

            if "humidity" in data:
                statsd.gauge(label + '.humidity', data["humidity"])

            statsd.gauge(label + '.temperature', data["temperature_C"])

        except KeyError:
            pass

        except ValueError:
            pass


if __name__ == "__main__":
    rtl_433_probe()



#!/usr/bin/env python3

"""Custom data handling example for rtl_433's HTTP WebSocket API of JSON events."""

# Start rtl_433 (`rtl_433 -F http`), then this script.
# Needs the websocket-client package to be installed.

import websocket
import json
from time import sleep

# You can run rtl_433 and this script on different machines,
# start rtl_433 with `-F http:0.0.0.0`, and change
# to e.g. `HTTP_HOST = "192.168.1.100"` (use your server ip) below.
HTTP_HOST = "127.0.0.1"
HTTP_PORT = 8433


def ws_events():
    url = f'ws://{HTTP_HOST}:{HTTP_PORT}/ws'
    ws = websocket.WebSocket()
    ws.connect(url)

    # You will receive JSON events, one per message.
    print(f'Connected to {url}')

    while True:
        yield ws.recv()


def handle_event(line):
    try:
        # Decode the message as JSON
        data = json.loads(line)

        #
        # Change for your custom handling below, this is a simple example
        #
        label = data["model"]
        if "channel" in data:
            label += ".CH" + str(data["channel"])
        elif "id" in data:
            label += ".ID" + str(data["id"])

        # E.g. match `model` and `id` to a descriptive name.
        if data["model"] == "LaCrosse-TX" and data["id"] == 123:
            label = "Living Room"

        if "battery_ok" in data:
            if data["battery_ok"] == 0:
                print(label + ' Battery empty!')

        if "temperature_C" in data:
            print(label + ' Temperature ', data["temperature_C"])

        if "humidity" in data:
            print(label + ' Humidity ', data["humidity"])

    except KeyError:
        # Ignore unknown message data and continue
        pass

    except ValueError as e:
        # Warn on decoding errors
        print(f'Event format not recognized: {e}')


def rtl_433_listen():
    """Listen to all messages in a loop forever."""

    # Loop forever
    while True:
        try:
            # Open the HTTP WebSocket API of JSON events
            for chunk in ws_events():
                # print(chunk)
                chunk = chunk.rstrip()
                if not chunk:
                    # filter out keep-alive empty lines
                    continue
                # Decode the JSON message
                handle_event(chunk)

        except ConnectionRefusedError:
            print('Connection refused, retrying...')
            sleep(5)
            pass

        except websocket._exceptions.WebSocketConnectionClosedException:
            print('Connection failed, retrying...')
            sleep(5)


if __name__ == "__main__":
    try:
        rtl_433_listen()
    except KeyboardInterrupt:
        print('\nExiting.')
        pass



#!/usr/bin/env python3

"""InfluxDB monitoring relay for rtl_433."""

# Start rtl_433 (rtl_433 -F syslog::1433), then this script

# Option: PEP 3143 - Standard daemon process library
# (use Python 3.x or pip install python-daemon)
# import daemon

from __future__ import print_function
from __future__ import with_statement

from influxdb import InfluxDBClient
import socket
from datetime import datetime
import json
import sys

UDP_IP = "127.0.0.1"
UDP_PORT = 1433
INFLUXDB_HOST = "127.0.0.1"
INFLUXDB_PORT = 8086
INFLUXDB_USERNAME = ""
INFLUXDB_PASSWORD = ""
INFLUXDB_DATABASE = "rtl433"

TAGS = [
    "channel",
    "id",
]

FIELDS = [
    "temperature_C",
    "humidity",
    "battery_ok",
    "pressure_hPa",
]

sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
sock.bind((UDP_IP, UDP_PORT))


def sanitize(text):
    return text.replace(" ", "_").replace("/", "_").replace(".", "_").replace("&", "")


def parse_syslog(line):
    """Try to extract the payload from a syslog line."""
    line = line.decode("ascii")  # also UTF-8 if BOM
    if line.startswith("<"):
        # fields should be "<PRI>VER", timestamp, hostname, command, pid, mid, sdata, payload
        fields = line.split(None, 7)
        line = fields[-1]
    return line


def rtl_433_probe():
    client = InfluxDBClient(host=INFLUXDB_HOST, port=INFLUXDB_PORT,
                            username=INFLUXDB_USERNAME, password=INFLUXDB_PASSWORD,
                            database=INFLUXDB_DATABASE)

    while True:
        line, _addr = sock.recvfrom(1024)

        try:
            line = parse_syslog(line)
            data = json.loads(line)

            if not "model" in data:
                continue
            measurement = sanitize(data["model"])

            tags = {}
            for tag in TAGS:
                if tag in data:
                    tags[tag] = data[tag]

            fields = {}
            for field in FIELDS:
                if field in data:
                    fields[field] = data[field]

            if len(fields) == 0:
                continue

            point = {
                "measurement": measurement,
                "time": datetime.now().isoformat(),
                "tags": tags,
                "fields": fields,
            }

            try:
                client.write_points([point])
            except Exception as e:
                print("error {} writing {}".format(e, point), file=sys.stderr)

        except KeyError:
            pass

        except ValueError:
            pass


def run():
    # with daemon.DaemonContext(files_preserve=[sock]):
    #  detach_process=True
    #  uid
    #  gid
    #  working_directory
    rtl_433_probe()


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
""" MQTT republishing filter to rename and deduplicate rtl_433 data

Example program for receiving, filtering and republishing sensor data
from rtl_433 using MQTT network messages.

An MQTT broker e.g. 'mosquitto' must be running and rtl_433 must publish
to that broker, e.g. using -F mqtt.
"""

import json
import logging
import sys
import time
import socket
import re

import paho.mqtt.client as mqtt
from paho.mqtt.subscribeoptions import SubscribeOptions

HOSTNAME = socket.getfqdn()

MQTT_SERVER = "127.0.0.1"  # set your MQTT broker address here

MQTT_TOPIC_PREFIX = "rtl_433/+"  # default to use all rtl_433 senders
# MQTT_TOPIC_PREFIX = "rtl_433/" + HOSTNAME  # alternative: use just the local host topics
# MQTT_TOPIC_PREFIX = "rtl_433/MYSERVER"  # alternativ: use a named rtl_433 sender

MQTT_TOPIC_DEVICES = MQTT_TOPIC_PREFIX + "/devices"  # default "devices" topic tree base
MQTT_TOPIC_EVENTS = MQTT_TOPIC_PREFIX + "/events"  # default "events" topic

# set source and target topics as well as deduplication in seconds, 0 disables deduplication
DEVICE_MAPPINGS = [
    { "source": "Bresser-3CH/1/130", "target": "Livingroom", "dedup": 1.5},
    { "source": "Bresser-3CH/2/175", "target": "Bedroom", "dedup": 1.5},
    { "source": "LaCrosse-TX141THBv2/2/122", "target": "Garage", "dedup": 0},
]

EVENT_MAPPINGS = [
    { "source": { "model": "Bresser-3CH", "id": 130 }, "target": { "nickname": "Livingroom" }, "dedup": 1.5},
    { "source": { "model": "Bresser-3CH", "id": 175 }, "target": { "nickname": "Bedroom" }, "dedup": 1.5},
    { "source": { "model": "LaCrosse-TX141THBv2", "id": 122 }, "target": { "nickname": "Garage" }, "dedup": 0},
]

EVENT_REPUBLISHED_KEY = "republished"  # a JSON key only present in republished messages
# EVENT_REPUBLISHED_KEY = "nickname"  # can also be key added with "target"

if hasattr(mqtt, 'CallbackAPIVersion'):  # paho >= 2.0.0
    mqtt_client = mqtt.Client(callback_api_version=mqtt.CallbackAPIVersion.VERSION1, client_id="RTL_433_Filter")
else:
    mqtt_client = mqtt.Client(client_id="RTL_433_Filter")

DEVICES_RE = MQTT_TOPIC_DEVICES.replace("+", ".+")
EVENTS_RE = MQTT_TOPIC_EVENTS.replace("+", ".+")

dedup_cache = {}


def publish_dedup(topic, payload, timeout):
    """ Deduplicate and publish a message. """
    global dedup_cache
    global mqtt_client

    if timeout <= 0:
        logging.debug("republishing " + topic + " : " + str(payload))
        mqtt_client.publish(topic, payload)
        return

    key = topic + str(payload)
    now = time.time()
    if key in dedup_cache and dedup_cache[key] > now:
        logging.info("dedup " + topic + " : " + str(payload))
    else:
        logging.debug("republishing " + topic + " : " + str(payload))
        mqtt_client.publish(topic, payload)

    dedup_cache[key] = now + timeout
    for k in list(dedup_cache.keys()):
        if dedup_cache[k] < now:
            del dedup_cache[k]


def filter_devices(topic, payload):
    """ Deduplicate and republish device messages. """
    global mqtt_client

    if not re.match(DEVICES_RE, topic):
        return

    # Loop through all device mappings
    for map in DEVICE_MAPPINGS:
        path = "(" + DEVICES_RE + "/)" + map["source"] + "(.+)"
        m = re.match(path, topic)
        # On match republish same payload to different topic
        if m:
            target = m.group(1) + map["target"] + m.group(2)
            logging.debug("republishing " + map["source"] + " : " + target)
            publish_dedup(target, payload, map["dedup"])


def is_sub_dict(needle, haystack):
    """ Test if all key-value pairs of needle match haystack. """
    for k, v in needle.items():
        if k not in haystack or haystack[k] != v:
            return False
    return True


def filter_events(topic, payload):
    """ Deduplicate and republish event messages. """
    global mqtt_client

    if not re.match(EVENTS_RE, topic):
        return

    try:
        data = json.loads(payload.decode())

        # MQTT v5 noLocal might not work
        # Ignore republished events, there should be a better way?
        if EVENT_REPUBLISHED_KEY in data:
            return

        # Loop through all event mappings
        for map in EVENT_MAPPINGS:
            # Ensure all JSON data keys match
            if not is_sub_dict(map["source"], data):
                continue
            # On match republish modified payload to same topic

            # Tag as repbulished event
            data[EVENT_REPUBLISHED_KEY] = 1
            # Add all "target" keys, e.g. a nickname
            for k, v in map["target"].items():
                data[k] = v
            logging.debug("republishing to " + topic)
            publish_dedup(topic, json.dumps(data), map["dedup"])

    except json.decoder.JSONDecodeError:
        logging.warning("JSON decode error: " + payload.decode())


def on_connect(client, userdata, flags, rc):
    """ Callback for when the client receives a CONNACK response from the server. """
    logging.info("MQTT Connection: " + mqtt.connack_string(rc))
    if rc != 0:
        logging.error("Could not connect. RC: " + str(rc))
        exit()
    # Subscribing in on_connect() means that if we lose the connection and reconnect then subscriptions will be renewed.
    options = SubscribeOptions(qos=1, noLocal=True)
    logging.info("Subscribing to " + MQTT_TOPIC_EVENTS)
    client.subscribe(MQTT_TOPIC_EVENTS, options=options)
    logging.info("Subscribing to " + MQTT_TOPIC_DEVICES + "/#")
    client.subscribe(MQTT_TOPIC_DEVICES + "/#", options=options)


def on_disconnect(client, userdata, rc):
    if rc != 0:
        logging.error("Unexpected disconnection. RC: " + str(rc))


def on_message(client, userdata, msg):
    """ Callback for when a PUBLISH message is received from the server. """
    logging.debug("Received: " + msg.topic + "\t" + msg.payload.decode())
    filter_devices(msg.topic, msg.payload)
    filter_events(msg.topic, msg.payload)


# Setup MQTT client
mqtt_client.on_connect = on_connect
mqtt_client.on_disconnect = on_disconnect
mqtt_client.on_message = on_message
mqtt_client.connect(MQTT_SERVER)
mqtt_client.loop_start()


def main():
    """MQTT republishing filter"""
    logging.basicConfig(format='[%(asctime)s] %(levelname)s:%(name)s:%(message)s',datefmt='%Y-%m-%dT%H:%M:%S%z')
    logging.getLogger().setLevel(logging.INFO)

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""RRDtool monitoring relay for rtl_433."""

# Start rtl_433 (rtl_433 -C si -F syslog:127.0.0.1:1433), then this script

from __future__ import print_function
from __future__ import with_statement

import sys
import socket
import time
import json
import rrdtool

# Option: PEP 3143 - Standard daemon process library
# (pip install python-daemon)
try:
    import daemon
except ImportError:
    daemon = None

UDP_IP = "127.0.0.1"
UDP_PORT = 1433
RRD_PATH = "" # e.g. "/var/lib/rtl_433/rrd/"
GRAPH_PATH = ""  # e.g. "/var/www/rrd/html/"
GRAPH_INTERVAL = 30 * 60  # in seconds, i.e. 30 minutes


def create_rrd(rrdfile):
    # print("Creating", rrdfile)
    return rrdtool.create(rrdfile,
                          "--step", "1800", "--start", '0',
                          "DS:temperature:GAUGE:2000:U:U",
                          "DS:humidity:GAUGE:2000:U:U",
                          "RRA:AVERAGE:0.5:1:600",
                          "RRA:AVERAGE:0.5:6:700",
                          "RRA:AVERAGE:0.5:24:775",
                          "RRA:AVERAGE:0.5:288:797",
                          "RRA:MAX:0.5:1:600",
                          "RRA:MAX:0.5:6:700",
                          "RRA:MAX:0.5:24:775",
                          "RRA:MAX:0.5:444:797")


def update_rrd(rrdfile, temperature, humidity):
    # print("Updating", rrdfile, temperature, humidity)
    return rrdtool.update(rrdfile, "N:%s:%s" % (temperature, humidity))


def graph_rrd(rrdfile, label, path=""):
    for sched in ['daily' , 'weekly', 'monthly', 'hourly']:
        period = sched[0] # 'w', 'd', 'm', 'h'
        # print("Graphing", sched, label, rrdfile)
        ret = rrdtool.graph("%smetrics-%s.%s.png" % (path, sched, label),
                            "--start", "-1%s" % (period),
                            "--title", label,
                            "--vertical-label=C",
                            "--right-axis-label=%",
                            '--watermark=rtl_433',
                            "-w 800", "-h 200",
                            "DEF:t=%s:temperature:AVERAGE" % (rrdfile),
                            "DEF:h=%s:humidity:AVERAGE" % (rrdfile),
                            "LINE1:t#00FF00:temperature\r",
                            "LINE2:h#0000FF:humidity\r",
                            "GPRINT:t:AVERAGE:Temp avg %6.1lf C",
                            "GPRINT:t:MAX:Temp max %6.1lf C\r",
                            "GPRINT:h:AVERAGE:Hum avg %6.0lf %%",
                            "GPRINT:h:MAX:Hum max %6.0lf %%\r")
    return ret


def sanitize(text):
    return text.replace(" ", "_").replace("/", "_").replace(".", "_").replace("&", "")


def parse_syslog(line):
    """Try to extract the payload from a syslog line."""
    line = line.decode("ascii")  # also UTF-8 if BOM
    if line.startswith("<"):
        # fields should be "<PRI>VER", timestamp, hostname, command, pid, mid, sdata, payload
        fields = line.split(None, 7)
        line = fields[-1]
    return line


def rtl_433_probe(sock):
    next_graph = {}

    while True:
        line, _addr = sock.recvfrom(1024)

        try:
            line = parse_syslog(line)
            data = json.loads(line)
            now = int(time.time())

            label = sanitize(data["model"])
            if "channel" in data:
                label += ".CH" + str(data["channel"])
            elif "id" in data:
                label += ".ID" + str(data["id"])
            rrd_file = RRD_PATH + label + ".rrd"

            if "type" in data and data["type"] == "TPMS":
                continue

            if "temperature_C" not in data:
                continue
            temperature = data["temperature_C"]

            humidity = "U"
            if "humidity" in data:
                humidity = data["humidity"]

            try:
                rrdtool.info(rrd_file)
            except rrdtool.OperationalError:
                create_rrd(rrd_file)

            update_rrd(rrd_file, temperature, humidity)

            if label not in next_graph or next_graph[label] < now:
                graph_rrd(rrd_file, label, GRAPH_PATH)
                next_graph[label] = now + GRAPH_INTERVAL

        except KeyError:
            pass

        except ValueError:
            pass


def run(run_as_daemon=False):
    # check optional python imports
    if run_as_daemon and not daemon:
        print("Error: pip install python-daemon")
        return

    # setup input and output
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
    sock.bind((UDP_IP, UDP_PORT))

    # run, as daemon if requested
    if run_as_daemon:
        with daemon.DaemonContext(files_preserve=[sock]):
            rtl_433_probe(sock)
    else:
        rtl_433_probe(sock)


if __name__ == "__main__":
    # simple argument parsing
    run_as_daemon = len(sys.argv) > 1 and sys.argv[1] == "-d"
    try:
        run(run_as_daemon)
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python3

"""Collectd monitoring probe (i.e. no plugin) for rtl_433."""

# Needs Python collectd Network plugin, s.a. https://github.com/appliedsec/collectd
#   pip install collectd
# -or-
#   curl -O https://github.com/appliedsec/collectd/raw/master/collectd.py

import time
import socket
import fileinput
import json
import collectd


def send_stats(when, stats, sender, to):
    for (plugin_type, plugin_inst), values in stats.items():
        if not values:
            continue
        collectd.PLUGIN_TYPE = plugin_type
        for message in collectd.messages(values, when, sender, plugin_inst):
            collectd.sock.sendto(message, to)


def sanitize(text):
    return text.replace(" ", "_")


def rtl_433_probe():
    hostname = socket.getfqdn()
    interval = 60.0  # seconds

    collectd.SEND_INTERVAL = interval
    collectd.PLUGIN_NAME = 'rtlsdr'

    collectd_host = "localhost"
    collectd_port = 25826

    for line in fileinput.input():
        try:
            data = json.loads(line)

            when = int(time.time())
            label = sanitize(data["model"])
            if "channel" in data:
                label += ".CH" + str(data["channel"])
            attributes = {}
            temperatures = {}

            attributes["battery"] = data["battery_ok"]

            attributes["humidity"] = data["humidity"]

            temperatures["sensor"] = data["temperature_C"]

            stats = {('gauge', label): attributes,
                     ('temperature', label): temperatures}

            send_stats(when, stats, hostname, (collectd_host, collectd_port))

        except ValueError:
            pass


if __name__ == "__main__":
    rtl_433_probe()



#!/usr/bin/env python3

"""
Prometheus/OpenMetrics relay for rtl_433.

Can either take in JSON from stdin:

rtl_433 ... -F json | examples/rtl_433_prometheus_relay.py

or through UDP syslog packets:

rtl_433 ... -F syslog:0:4433
examples/rtl_433_prometheus_relay.py 4433

Then have Prometheus scrape it on port 30584, or change _SRV_PORT below.

Built for and tested with Oregon Scientific weather sensors, but *should*
work for other kinds (but only for numeric data).
"""

import collections
import json
import os
import socket
import sys
import threading
import time

import dateutil.parser
import http.server

## 0x77 w 0x78 x -> port 30584
_SRV_PORT = 0x7778


class rtl_433(object):
    # These fields are used to separate and tag unique sensors, instead of being exported
    # as readings/values.
    _ID_FIELDS = [
        ("model", str),
        ("type", str),
        ("subtype", str),
        ("channel", int),
        ("id", int),
    ]
    _MIN_REPEAT_SECS = 3600  # See a device at least twice in this timespan before sharing it.
    _EXPORT_TIMESTAMPS = True

    _MAX_AGE_SECS = 300  # Used for gc, and max age to show *anything* for given id
    _BACKLOG_SECS = 60   # If multiple samples within this timestamp, share them all

    _LOG_CLEAN_INTERVAL = 1000  # Number of iterations

    log = []  # [(timestamp, id_fields, variable name, value), ...]

    def __init__(self, stream):
        self.stream = stream

    def loop(self):
        n = 0
        last = {}
        for line in self.stream:
            now = time.time()
            print(line.strip())
            try:
                pkt = json.loads(line)
            except json.decoder.JSONDecodeError as e:
                print("Parse error: %s" % e.msg)
                continue
            if not isinstance(pkt, dict):
                print("Not a dict/object: %r" % pkt)
                continue
            try:
                ts = dateutil.parser.isoparse(pkt.pop("time")).timestamp()
            except KeyError:
                ts = now
            if int(ts) == ts:
                # Check above in case we start getting fed non-integer timestamps already.
                # If not, use current precise time instead.
                if abs(now - (ts + 0.5)) < 1:
                    ts = now

            id = self.grab_id(pkt)
            ago = ts - last.get(id, 0)
            last[id] = ts
            if ago < self._MIN_REPEAT_SECS:
                new = []
                for k, v in pkt.items():
                    try:
                        new.append((ts, id, k, float(v)))
                    except (ValueError, TypeError):
                        print("%s{%r} has non-numeric value %s" % (k, id, v))
                        continue
                self.log += new

            n += 1
            if (n % self._LOG_CLEAN_INTERVAL) == 0:
                self.clean_log()

    @staticmethod
    def grab_id(pkt):
        ret = []
        for field, ftype in rtl_433._ID_FIELDS:
            v = pkt.pop(field, None)
            try:
                v = ftype(v)
            except (ValueError, TypeError):
                pass  # Stick to existing type
            ret.append(v)
        return tuple(ret)

    def metrics(self):
        ret = collections.defaultdict(list)
        now = time.time()
        seen = {}  # value should be most recent timestamp this id has sent data for.
        for ts, id, var, val in reversed(self.log):
            if now - ts > self._MAX_AGE_SECS:
                break
            if now - ts > self._BACKLOG_SECS and ts < seen.get(id, 0):
                continue
            if id not in seen:
                seen[id] = ts

            id_s = ",".join(("%s=\"%s\"" % (f[0], v)) for f, v in zip(self._ID_FIELDS, id) if v is not None)
            if self._EXPORT_TIMESTAMPS:
                ret[var].append("%s{%s} %f %d" % (var, id_s, val, ts * 1000))
            elif ts == seen[id]:
                ret[var].append("%s{%s} %f" % (var, id_s, val))

        return ("\n".join("\n".join(lines) for lines in ret.values())) + "\n"

    def clean_log(self):
        if not self.log:
            return
        if self.log[0][0] - self.log[-1][0] > 60:
            # Time has gone the wrong way by a long time. Drop the table.
            self.log = []
            return

        min_ts = time.time() - self._MAX_AGE_SECS
        for i, e in enumerate(self.log):
            if e[0] >= min_ts:
                break

        # I think this should be safe even if we're serving /metrics since
        # I'm replacing not modifying the log.
        self.log = self.log[i:]


def syslog_reader(udp_port, udp_ip="0.0.0.0"):
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
    sock.bind((udp_ip, udp_port))
    while True:
        pkt, _ = sock.recvfrom(4096)
        bits = pkt.split(None, 7)
        if len(bits) == 8:
            yield str(bits[7], "utf-8")


if len(sys.argv) == 1:
    r = rtl_433(sys.stdin)
else:
    r = rtl_433(syslog_reader(int(sys.argv[1])))


class MetricsHandler(http.server.BaseHTTPRequestHandler):
    def do_GET(self):
        # boilerplate-- :<
        self.send_response(200)
        self.send_header("Content-Type", "text/plain; charset=utf-8")
        self.end_headers()
        self.wfile.write(r.metrics().encode("utf-8"))

class MetricsServer(http.server.HTTPServer):
    def handle_error(self, req, addr):
        super().handle_error(req, addr)
        os._exit(6)

https = MetricsServer(("0.0.0.0", _SRV_PORT), MetricsHandler)
httpd = threading.Thread(name="httpd", target=https.serve_forever, daemon=True)
httpd.start()


r.loop()



#!/usr/bin/env python3

"""MQTT monitoring relay for rtl_433 communication."""

# This program listens on a UDP socket for syslog messages with a json
# payload, and publishes the data via MQTT.  The broker connection is
# kept open (and automatically reconnects on failure).  Each device
# is mapped to its own topic,

# Dependencies:
#   Paho-MQTT; see https://pypi.python.org/pypi/paho-mqtt

#   Optionally: PEP 3143 - Standard daemon process library
#      (on 2.7,  pip install python-daemon)

# To enable daemon support, uncomment the following line and adjust
# run().  Note that print() is still used.
# import daemon

from __future__ import print_function
from __future__ import with_statement

import json
import logging
import socket
import time

import paho.mqtt.client as mqtt


# The config class represents a config object.  The constructor takes
# an optional pathname, and will switch on the suffix (.yaml for now)
# and read a dictionary.
class rtlconfig(object):

    # Initialize with default values.
    c = {
        # Log level info (False) or debug (True)
        'DEBUG': False,

        # Address to listen on for syslog/json messages from rtl_433
        'UDP_IP': "127.0.0.1",
        'UDP_PORT': 1433,
        
        # MQTT broker address and credentials
        'MQTT_HOST': "127.0.0.1",
        'MQTT_PORT': 1883,
        'MQTT_USERNAME': None,
        'MQTT_PASSWORD': None,
        'MQTT_TLS': False,

        # MQTT content
        'MQTT_PREFIX': "sensor/rtl_433",
        'MQTT_DEDUP': True,
        'MQTT_INDIVIDUAL_TOPICS': True,
        'MQTT_JSON_TOPIC': True,
    }
    
    def __init__(self, f=None):
        fdict = None

        # Try to read a dictionary from f.
        if f:
            try:
                # Assume yaml. \todo Check and support other formats
                import yaml
                with open(f) as fh:
                    fdict = yaml.safe_load(fh)
            except:
                print('Did not read {f} (no yaml, not found, bad?).'.format(f=f))
            
        # Merge fdict into configdict.
        if fdict:
            for (k, v) in fdict.items():
                self.c[k] = v

    # Support c['name'] references.
    def __getitem__(self, k):
        return self.c[k]

class dedup(object):
    """ A dedup class object supports deduping a stream of reports by
        answering if a report is interesting relative to the history.  While
        more complicated deduping is allowed by the interface, for now it is
        very simple, keeping track of only the previous interesting object.
        For now, we more or less require that all reports have the same keys. """

    # \todo Consider a cache with several entries.

    def __init__(self):
        # Make this long enough to skip repeats, but allow messages
        # every 10s to come through.
        self.duration = 5
        # Exclude reception metadata (time and RF).
        self.boring_keys = ('time', 'freq', 'freq1', 'freq2', 'rssi', 'snr', 'noise', 'raw_msg')
        # Initialize storage for what was last sent.
        (self.last_report, self.last_now) = (None, None)
    
    def send_store(self, report, n):
        """ Record report, n as the last report declared interesting, and
            return True (to denote interesting). """
        (self.last_report, self.last_now) = (report, n)
        return True

    def equiv(self, j1, j2):
        """ Return True if j1 and j2 are the same, except for boring_keys. """
        for (k, v) in j1.items():
            # If in boring, we don't care.
            if k not in self.boring_keys:
                # If in j1 and not j2, they are different.
                if k not in j2:
                    logging.debug("equiv: %s in j1 and not j2" % (k))
                    return False
                if j1[k] != j2[k]:
                    logging.debug("equiv: %s differs j1=%s and j2=%s" % (k, j1[k], j2[k]))
                    return False
        # If the lengths are different, they must be different.
        if len(j1) != len(j2): 
            logging.debug("equiv: len(j1) %d != len(j2) %d" % (len(j1), len(j2)))
            return False

        # If we get here, then the lengths are the same, and all
        # non-boring keys in j1 exist in j2, and have the same value.
        # It could be that j2 is missing a boring key and also has a
        # new non-boring key, but boring keys in particular should not
        # be variable.
        return True

    # report is a python dictionary
    def is_interesting(self, report):
        """ If report is intersting, return True and update records of the
            most recent interesting report.  Otherwise return False. """
        n = time.time()

        # If previous interesting is missing or empty, accept this one.
        if self.last_report is None or self.last_now is None:
            logging.debug("interesting: no previous")
            return self.send_store(report, n)

        # If previous one was too long ago, accept this one.
        if n - self.last_now > self.duration:
            logging.debug("interesting: time")
            return self.send_store(report, n)

        if not self.equiv(self.last_report, report):
            logging.debug("interesting: different")
            return self.send_store(report, n)

        return False

# Create a config object, defaults modified by the config file if present.
c = rtlconfig("rtl_433_mqtt_relay.yaml")

# Create a dedup object for later use, even if it's configured off.
d = dedup()

def mqtt_connect(client, userdata, flags, rc):
    """Handle MQTT connection callback."""
    logging.info("MQTT connected: " + mqtt.connack_string(rc))


def mqtt_disconnect(client, userdata, rc):
    """Handle MQTT disconnection callback."""
    logging.info("MQTT disconnected: " + mqtt.connack_string(rc))


# Create listener for incoming json string packets.
sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
sock.bind((c['UDP_IP'], c['UDP_PORT']))


# Map characters that will cause problems or be confusing in mqtt
# topics.
def sanitize(text):
    """Sanitize a name for Graphite/MQTT use."""
    return (text
            .replace(" ", "_")
            .replace("/", "_")
            .replace(".", "_")
            .replace("&", ""))


def publish_sensor_to_mqtt(mqttc, data, line):
    """Publish rtl_433 sensor data to MQTT."""

    if c['MQTT_DEDUP']:
        # If this data is not novel relative to recent data, just skip it.
        # Otherwise, send it via MQTT.
        if not d.is_interesting(data):
            logging.debug("  not interesting")
            return
        logging.debug(  "INTERESTING")

    # Construct a topic from the information that identifies which
    # device this frame is from.
    # NB: id is only used if channel is not present.
    path = c['MQTT_PREFIX']
    if "model" in data:
        path += "/" + sanitize(data["model"])
    if "channel" in data:
        path += "/" + str(data["channel"])
    if "id" in data:
        path += "/" + str(data["id"])

    if c['MQTT_INDIVIDUAL_TOPICS']:
        # Publish some specific items on subtopics.
        if "battery_ok" in data:
            mqttc.publish(path + "/battery", data["battery_ok"])

        if "humidity" in data:
            mqttc.publish(path + "/humidity", data["humidity"])

        if "temperature_C" in data:
            mqttc.publish(path + "/temperature", data["temperature_C"])

        if "depth_cm" in data:
            mqttc.publish(path + "/depth", data["depth_cm"])

    if c['MQTT_JSON_TOPIC']:
        # Publish the entire json string on the main topic.
        mqttc.publish(path, line)

def parse_syslog(line):
    """Try to extract the payload from a syslog line."""
    line = line.decode("ascii")  # also UTF-8 if BOM
    if line.startswith("<"):
        # Fields should be "<PRI>VER", timestamp, hostname, command, pid, mid, sdata, payload.
        # The payload might have spaces, so force split to stop after the sixth space.
        fields = line.split(None, 7)
        line = fields[-1]
    else:
        # Hope that the line was just json without the syslog header.
        pass
    return line


def rtl_433_probe():
    """Run a rtl_433 UDP listener."""

    ## Connect to MQTT
    if hasattr(mqtt, 'CallbackAPIVersion'):  # paho >= 2.0.0
        mqttc = mqtt.Client(callback_api_version=mqtt.CallbackAPIVersion.VERSION1)
    else:
        mqttc = mqtt.Client()
    mqttc.on_connect = mqtt_connect
    mqttc.on_disconnect = mqtt_disconnect
    if c['MQTT_USERNAME'] != None:
        mqttc.username_pw_set(c['MQTT_USERNAME'], password=c['MQTT_PASSWORD'])
    if c['MQTT_TLS']:
        mqttc.tls_set()
    mqttc.connect_async(c['MQTT_HOST'], c['MQTT_PORT'], 60)
    mqttc.loop_start()

    ## Receive UDP datagrams, extract json, and publish.
    while True:
        line, addr = sock.recvfrom(1024)
        try:
            line = parse_syslog(line)
            data = json.loads(line)
            logging.debug("received %s" % line)
            publish_sensor_to_mqtt(mqttc, data, line)

        except ValueError:
            pass


def run():
    """Run main or daemon."""
    # with daemon.DaemonContext(files_preserve=[sock]):
    #  detach_process=True
    #  uid
    #  gid
    #  working_directory

    # Set up logging at INFO, and change to DEBUG if config asks for that.
    logging.basicConfig(format='[%(asctime)s] %(levelname)s:%(name)s:%(message)s',datefmt='%Y-%m-%dT%H:%M:%S%z')
    logging.getLogger().setLevel(logging.INFO)
    if c['DEBUG']:
        logging.getLogger().setLevel(logging.DEBUG)
        logging.debug("DEBUG LOGGING ENABLED")

    rtl_433_probe()

if __name__ == "__main__":
    run()
